#
# gtkui.py
#
# Copyright (C) 2009 Rune K. Svendsen <runesvend@gmail.com>
#
# Basic plugin template created by:
# Copyright (C) 2008 Martijn Voncken <mvoncken@gmail.com>
# Copyright (C) 2007-2009 Andrew Resch <andrewresch@gmail.com>
# Copyright (C) 2009 Damien Churchill <damoxc@gmail.com>
#
# Deluge is free software.
#
# You may redistribute it and/or modify it under the terms of the
# GNU General Public License, as published by the Free Software
# Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# deluge is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with deluge.    If not, write to:
# 	The Free Software Foundation, Inc.,
# 	51 Franklin Street, Fifth Floor
# 	Boston, MA  02110-1301, USA.
#
#    In addition, as a special exception, the copyright holders give
#    permission to link the code of portions of this program with the OpenSSL
#    library.
#    You must obey the GNU General Public License in all respects for all of
#    the code used other than OpenSSL. If you modify file(s) with this
#    exception, you may extend this exception to your version of the file(s),
#    but you are not obligated to do so. If you do not wish to do so, delete
#    this exception statement from your version. If you delete this exception
#    statement from all source files in the program, then also delete it here.
#

import gtk

from deluge.log import LOG as log
from deluge.ui.client import client
from deluge.plugins.pluginbase import GtkPluginBase
import deluge.component as component
import deluge.common

from common import get_resource

from urllib import quote_plus
import pyperclip #for cross platform clipboard support
import os

class GtkUI(GtkPluginBase):
    def enable(self):
        self.glade = gtk.glade.XML(get_resource("config.glade"))

        component.get("Preferences").add_page("CopyMagnetURI", self.glade.get_widget("prefs_box"))
        component.get("PluginManager").register_hook("on_apply_prefs", self.on_apply_prefs)
        component.get("PluginManager").register_hook("on_show_prefs", self.on_show_prefs)

        torrentmenu = component.get("MenuBar").torrentmenu_glade.get_widget("torrent_menu")

        self.sep = gtk.SeparatorMenuItem()
        self.item = gtk.MenuItem(_("_Copy Magnet URI"))
        self.item.connect("activate", self.on_menuitem_copy_magnet_uri)
        
        torrentmenu.append(self.sep)
        torrentmenu.append(self.item)

        self.sep.show()
        self.item.show()
        

    def disable(self):
        component.get("Preferences").remove_page("CopyMagnetURI")
        component.get("PluginManager").deregister_hook("on_apply_prefs", self.on_apply_prefs)
        component.get("PluginManager").deregister_hook("on_show_prefs", self.on_show_prefs)

        torrentmenu = component.get("MenuBar").torrentmenu_glade.get_widget("torrent_menu")

        torrentmenu.remove(self.item)
        torrentmenu.remove(self.sep)

    def on_apply_prefs(self):
        log.debug("applying prefs for CopyMagnetURI")
        config = {
            "test":self.glade.get_widget("txt_test").get_text()
        }
        client.copymagneturi.set_config(config)

    def on_show_prefs(self):
        client.copymagneturi.get_config().addCallback(self.cb_get_config)

    def cb_get_config(self, config):
        "callback for on show_prefs"
        self.glade.get_widget("txt_test").set_text(config["test"])

    def on_menuitem_copy_magnet_uri(self, data=None):
        self.clipboard = ""
        def _on_torrent_status(status):
            hash = status["hash"]
            trackers = status["trackers"]
            name = status["name"]

            uri = self.format_uri(name, hash, trackers)
            self.clipboard += uri + os.linesep
            pyperclip.copy(self.clipboard)

        for torrent_id in component.get("TorrentView").get_selected_torrents():
            component.get("SessionProxy").get_torrent_status(torrent_id, ["hash", "trackers", "name"]).addCallback(_on_torrent_status)

    def format_uri(self, name, hash, trackers):
        uri = "magnet:?xt=urn:btih:{}&dn={}".format(hash, quote_plus(name.encode('utf8')))
        tracker_format = "&tr={}" 
        for tracker in trackers:
            uri += tracker_format.format(quote_plus(tracker['url']))
        return uri
